// -*- mode:c++ -*-
//
// Module LookaheadContext.cc
//
// Copyright (c) 2001 Raoul M. Gough
//
// History
// =======
// 2001/ 7/28	rmg	File creation
//

#include "LookaheadContext.hh"

#include <limits.h>
#include "utils.hh"

using std::ostream;
using std::endl;

LookaheadContext::LookaheadContext (void)
  : mFlagPos (SMLState::FlagCount)
  , mUnderlinePos (SMLState::UnderlineCount)
  , mSizePos (SMLState::SizeCount)
  , mColourPos (SMLState::ColourCount)
{
}

void LookaheadContext::add (SMLState prev, SMLState now, Position p)
{
  int flags = now.flagSet(); // & (~prev.flagSet());
  SMLState::Packed mask = 1;

  for (int count = 0; count < SMLState::FlagCount; ++count)
    {
      if (flags & mask)
	{
	  mFlagPos[count].push_back (p);
	}

      mask <<= 1;
    }

  if (now.underline() > 0)
    {
      mUnderlinePos[now.underline()].push_back (p);
    }

  if (now.hasSize())
    {
      mSizePos[now.size()].push_back (p);
    }

  if (now.hasColour())
    {
      mColourPos[now.colour()].push_back (p);
    }

  /*
  if (now.underline() != prev.underline())
    {
      mUnderlinePos[now.underline()].push_back (p);
    }

  if ((now.size() != prev.size()) && (now.hasSize()))
    {
      mSizePos[now.size()].push_back (p);
    }

  if ((now.colour() != prev.colour()) && now.hasColour())
    {
      mColourPos[now.colour()].push_back (p);
    }
  */
}

ostream& operator<< (ostream& strm, const LookaheadContext& c)
{
  for (int count = 0; count < SMLState::FlagCount; ++count)
    {
      strm << "Flag      " << count << ": " << c.mFlagPos[count].stack << endl;
    }

  for (int count = 0; count < SMLState::UnderlineCount; ++count)
    {
      strm << "Underline " << count << ": " << c.mUnderlinePos[count].stack << endl;
    }

  for (int count = 0; count < SMLState::SizeCount; ++count)
    {
      strm << "Size      " << count << ": " << c.mSizePos[count].stack << endl;
    }

  for (int count = 0; count < SMLState::ColourCount; ++count)
    {
      strm << "Colour " << count << ": " << c.mColourPos[count].stack << endl;
    }
}

void init (std::vector<LookaheadContext::PositionStack>& stackv)
{
  for (int count = 0; count < stackv.size(); ++count)
    {
      stackv[count].head = stackv[count].stack.rbegin();
    }
}

void LookaheadContext::initHeads (void)
{
  init (mFlagPos);
  init (mUnderlinePos);
  init (mSizePos);
  init (mColourPos);
}
