// -*- mode:c++ -*-
//
// Header file SMLLexer.hh
//
// Copyright (c) 2001 Raoul M. Gough
//
// *arg_yyout = input echo stream
// decorated = decorated output stream
//
// History
// =======
// 2001/ 7/28	rmg	File creation
//

#ifndef SMLLexer_included
#define SMLLexer_included

#include <iosfwd>
#include <vector>
#include <stack>
#include <FlexLexer.h>
#include "SMLState.hh"
#include "SMLStateStack.hh"
#include "LookaheadContext.hh"

class SMLLexer : public yyFlexLexer
{
public:
  SMLLexer (ostream& decorated
	    , istream* arg_yyin = 0
	    , ostream* arg_yyout = 0);

public:
  // Implementation provided by flex++
  virtual int yylex (void);

public:
  void dumpPosition (ostream& strm)
  {
    if (mInputPos == -1)
      {
	strm << "EOF";
      }

    else
      {
	strm << "input char " << mInputPos << " (line " << yylineno << ")";
      }
  }

public:
  const char *flagName (int flag);

public:
  const char *flagNumber (Tag);

public:
  inline std::vector<int>::const_iterator lookaheadIndexBegin()
  {
    return mContextList.begin();
  }

public:
  inline std::vector<int>::const_iterator lookaheadIndexEnd()
  {
    return mContextList.end();
  }

public:
  const LookaheadContext &getLookahead (int index) const
  {
    return mContext[index];
  }

public:
  const std::vector<LookaheadContext> &getLookahead (void) const
  {
    return mContext;
  }

public:
  LookaheadContext &getLookahead (int index)
  {
    return mContext[index];
  }

public:
  std::vector<LookaheadContext> &getLookahead (void)
  {
    return mContext;
  }

private:
  void syncState (SMLState);

private:
  void output (void);

private:
  inline void acc (void)
  {
    yyout->write (YYText(), YYLeng());
    mInputPos += YYLeng();
  }

private:
  void pushContext (void);

private:
  void popContext (void);

private:
  ostream&      mDecorated;
  SMLState      mState;
  SMLStateStack mStack;

  SMLState      mLastOutputState;
  bool          mOutput;
  bool          mSpace;
  bool          mColourOutput;
  bool          mSizeOutput;

  int           mInputPos;
  int           mOutputPos;

  std::vector<LookaheadContext> mContext;
  std::vector<int> mContextList;
  std::stack<int> mContextStack;
  LookaheadContext *mContextp;
  int               mCurrentContextIndex;
};

#endif // SMLLexer_included
