// -*- mode:c++ -*-
//
// Header file SMLStateStack.hh
//
// Copyright (c) 2001 Raoul M. Gough
//
// History
// =======
// 2001/ 7/28	rmg	File creation
//

#ifndef SMLStateStack_included
#define SMLStateStack_included

#include <deque>
#include <utility>
#include "Tag.hh"

typedef std::pair<Tag, SMLState::Packed> TagState;
typedef std::deque<TagState>             TagStateStack;

typedef std::pair<Tag, Tag>              TagMismatch;
typedef Tag                              StackEmpty;

struct SMLStateStack
{
  inline void push (Tag t, SMLState s)
  {
    mContainer.push_back (std::make_pair (t, s.pack()));
  }

  SMLState pop (Tag t)
  {
    if (mContainer.empty())
      {
	throw StackEmpty();
      }

    else if (mContainer.rbegin()->first != t)
      {
	throw TagMismatch (t, mContainer.rbegin()->first);
      }

    else
      {
	SMLState state (mContainer.rbegin()->second);
	mContainer.pop_back();
	return state;
      }
  }

  TagStateStack mContainer;
};

#endif // SMLStateStack_included
